﻿// $Header: /volume1/cvs/cvsroot/microrisc/iqrf/IqrfProjects/IQRFcode/UnitTestIQRFcode/UnitTestIQRFcode.cs,v 1.1 2018/03/22 18:26:16 hynek Exp $
namespace UnitTestIQRFcode
{
  using System;
  using System.Linq;
  using IQRFcode;
  using Microsoft.VisualStudio.TestTools.UnitTesting;

  /// <summary>
  /// Tests.
  /// </summary>
  [TestClass]
  public class UnitTestIQRFcode
  {
    [TestMethod]
    public void TestEncodeDecodeBasic ()
    {
      EncodeDecodeTest( new byte[] { 0 } );
      EncodeDecodeTest( new byte[] { 0xFF } );
      EncodeDecodeTest( new byte[] { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF } );
      EncodeDecodeTest( new byte[] { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF } );
      EncodeDecodeTest( new byte[] { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF } );

      for ( int length = 1; length <= 1024; length++ )
      {
        byte[] bytes = new byte[length];
        for ( int index = 0; index < bytes.Length; index++ )
          bytes[index] = (byte)( ( index * length + 1 ) & 0xFF );

        EncodeDecodeTest( bytes );
      }
    }

    [TestMethod]
    public void TestEncodeDecodeRandom ()
    {
      Random rand = new Random();

      for ( int count = 10000; --count != 0; )
      {
        int length = rand.Next( 1, 10000 );
        byte[] bytes = new byte[length];
        rand.NextBytes( bytes );

        EncodeDecodeTest( bytes );
      }
    }

    private void EncodeDecodeTest ( byte[] input )
    {
      string encoded = Base57.EncodeBytes( input );
      byte[] decoded = Base57.DecodeText( encoded );
      Assert.IsTrue( input.SequenceEqual( decoded ) );
    }

    [TestMethod]
    public void TestNibbles ()
    {
      NibbleWriter wr = new NibbleWriter();
      wr.WriteNibble( 1 );
      NibbleReader rd = new NibbleReader( wr.Close() );
      Assert.AreEqual( rd.ReadNibble(), 1 );

      wr = new NibbleWriter();
      wr.WriteNibble( 1 );
      wr.WriteByte( 0x23 );
      rd = new NibbleReader( wr.Close() );
      Assert.AreEqual( rd.ReadNibble(), 1 );
      Assert.AreEqual( rd.ReadByte(), 0x23 );

      wr = new NibbleWriter();
      wr.WriteNibble( 1 );
      wr.WriteByte( 0x23 );
      wr.WriteNibble( 4 );
      rd = new NibbleReader( wr.Close() );
      Assert.AreEqual( rd.ReadNibble(), 1 );
      Assert.AreEqual( rd.ReadByte(), 0x23 );
      Assert.AreEqual( rd.ReadNibble(), 4 );
    }
  }
}
